// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include "checksources.h"
#include "Dal.h"

#include "sourcelist.h"

#include "iostream"
#include <fstream>
#include <vector>
#include <map>
#include<iomanip>
#include"utility.h"
#include "statistics.h"
int numberOfNeighbours(int number, double x1, double y1, double minDist, 
		       vector<OmSourceParameter> &source,
                       vector<bool> & checked,
                       map<int, vector<OmSourceParameter> > &);


void toRegion(const vector<OmSourceParameter> & sources, 
              const char *colour, const bool showNumbers = true);


void computeStatistics(vector<OmSourceParameter> &data, float &median,
                      float &percentile, float frac = 0.95);


void getImageSize(string file, int &nx, int &ny);

int statistics(int nx, int ny, vector<OmSourceParameter> &source, int width, float minSig);

void spuriousSources(int nx, int ny, vector<OmSourceParameter> &source, int width, float minSig, int maxCount);

/*********************************************************************
*
*
*
**********************************************************************/
void checksources()
{

    string file = stringParameter( "set" );
    cout << "Checking file " << file << endl;


    vector<OmSourceParameter> source;
    int size = 0;
    string filter;
    float exposure;

    if ( !loadSourceParameters1( file, "SRCLIST", size, source, filter, exposure) )
    {
        cout << "Error loading data ferom file " << file << endl;

    }

    // Get the image size
    int nx, ny;
    getImageSize(file, nx, ny);


    int width = 24;
    float minSig = 5.0; 
    int p2 = statistics(nx, ny, source, width, minSig);

    spuriousSources(nx, ny, source, width, minSig, p2);


     exit(0);






    float median = 0.0;
    float percentile = 0.0;
    float frac = 0.1;
    computeStatistics(source, median, percentile, frac);
    cout << "Median separation=" << median << endl;
    cout << frac << " percentile=" << percentile << endl;
    vector<bool> checked(source.size(), false);
    map<int, int> mapCount;
    map<int, vector<OmSourceParameter> > mapSource;


    int number = 0;

    /********************************************************
    * Loop through all the sources and check for neighbours
    *
    ********************************************************/

    for(unsigned i = 0; i < source.size(); i++)
    {
      
	if(checked[i]) continue;
        checked[i] = true;

      
        if (source[i].significance > 10.0)continue;
        
      
   
	// Maybe spurious
        OmSourceParameter theSource = source[i];
        theSource.number = ++number;      
        mapSource[number].push_back( theSource );  
      
        // Check the neighbours to this source

        double x1 = source[i].xcoord;
        double y1 = source[i].ycoord;
       

        int count = numberOfNeighbours(number, x1, y1, percentile, source, checked, mapSource);

    
        ++mapCount[count];
	//        cout << "Source " << i << " has " << count << "  neighbours\n";  


        
    }


    map<int, int>::iterator iter;
    int sum = 0, count = 0;
    for(iter = mapCount.begin(); iter != mapCount.end(); iter++)
    {
      cout << "Number of neighbours " << iter->first << " frequency=" << iter->second << endl;
      sum += (iter->first * iter->second);
      count += iter->second;
    }
    cout << "Average number of neighbours per source = " << double(sum) / double(count) << endl; 

   
    vector<OmSourceParameter> spuriousSource;

    map<int, vector<OmSourceParameter> >::iterator iter1;

    for(iter1 = mapSource.begin(); iter1 != mapSource.end(); iter1++)
    {
        vector<OmSourceParameter> source = iter1->second;
        if(source.size() > 1)
	{
	    for(unsigned i = 0; i < source.size(); i++)
	      spuriousSource.push_back( source[i] );

        }


    }
    /**************************************************************
    * Create a region file
    **************************************************************/
 
    toRegion( spuriousSource, "red");

}
/*********************************************************
*
*
*
*
*********************************************************/
int numberOfNeighbours(int number, double x1, double y1, double minDist, 
		       vector<OmSourceParameter> &source,
                       vector<bool> & checked,
                       map<int, vector<OmSourceParameter> > &mapSource)

{
  
    int count = 0; // The number of neighbours to source at x1, y1

    for(unsigned i = 0; i < source.size(); i++)
    {
        if(checked[i]) continue; 
      
 
        double x2 = source[i].xcoord;
        double y2 = source[i].ycoord;
        double dx = x1 - x2;
        double dy = y1 - y2;

        double distance = sqrt(dx * dx + dy * dy);
        if(distance <= minDist)
        {
            checked[i] = true;
 

            if (source[i].significance <= 10.0)
	    {

	        // Possible spurious source- add it to the list
	        ++count;
                OmSourceParameter theSource = source[i];
                theSource.number = number;
                mapSource[number].push_back( theSource );  
   

            }          
         
            count += numberOfNeighbours(number, x2, y2, minDist, source, checked, mapSource);

            
        }
    }
    return count;

}
/***************************************************************
*
*
*
****************************************************************/
void toRegion(const vector<OmSourceParameter> & sources, 
	       const char *colour, const bool showNumbers)
{
       
  
	
    string regionFile = stringParameter("regionfile");  

    ofstream os( regionFile.c_str() );

    os << "global select=0 edit=0 move=0 delete=1" << endl;


    //     os << "global select=1 edit=1 move=1 delete=1" << endl;

    for (unsigned id = 0; id < sources.size(); id++) 
    {


        OmSourceParameter s = sources[id];   

      

        float semia = 1.0;
        float semib = 1.0;
        float theta = 0.0;  
	if( !isNull( s.semia ) )     
	{
	    semia = max(semia, s.semia);
            semib = max(semib, s.semib);
            theta = s.theta;
          
        }

  
        os << " fk5;ellipse("
	          << setprecision(6)  
		   << s.xcoord << ','
		   << s.ycoord << ','
		   << semia  << ','
		   << semib  << ','
		   << theta << ')';  // Position angel of ellipse 
	if ( showNumbers )
	{  
	  os  << " #text={" << s.number << "} color=" << colour;

	}
        else
	{

	    os  << " # color=" << colour;


        }
	os << endl;


	  






    }
   

    os.close();


}
/************************************************************
*
*
************************************************************/
void computeStatistics(vector<OmSourceParameter> &data, float &median, 
    float &percentile, float frac)
{


    vector<double> distance;
   

    for(unsigned i = 0; i < data.size(); i++)
    {
        if( data[i].significance > 10.0) continue;

        double x1 = data[i].xcoord;
        double y1 = data[i].ycoord;
       
        double minDistance = 1e9;        
        for(unsigned j = 0; j < data.size(); j++)
	{
	    if ( i == j ) continue;
            if( data[j].significance > 10.0) continue;
            
            double x2 = data[j].xcoord;
            double y2 = data[j].ycoord;
            double dx = x2 - x1;
            double dy = y2 - y1;
            double radius = sqrt(dx * dx + dy * dy);
            minDistance = min( minDistance, radius );
        }
        distance.push_back( minDistance );
    } 
 
    if( distance.size() > 1)
    {
      sort( distance.begin(), distance.end() );
      median = Median(distance);
      percentile = Percentile(distance, frac);
      for(unsigned i = 0; i < distance.size(); i++)
      {
	cout << i << ") " << setprecision(6) << distance[i] << endl;

      }

    }
    else
    {
      median = 0;

    }

      
    
}
/******************************************************************
*
******************************************************************/
void getImageSize(string file, int &nx, int &ny)
{

    DataSet *dataSet = dataSetServer->open( file, Dal::Read);
 





    //    cout << "Line 279\n";  


    if (!dataSet)
    {
   
        Sas::fatal("Could not load file-",  file );

    }

    bool binBpe = dataSet->attribute("BINBPE")->asBool();
   
    int winx = dataSet->attribute("WINDOWDX")->asInt() ;
 
    int winy = dataSet->attribute("WINDOWDY")->asInt() ;
 
 

    int binAx1 = (int)pow(2.0, dataSet->attribute("BINAX1")->asInt() );
 
    int binAx2 = (int)pow(2.0, dataSet->attribute("BINAX2")->asInt() );
 
    if(binBpe)
    {
      binAx1 *= 2;
      binAx2 *= 2;

    }
    nx = winx / binAx1;
    ny = winy / binAx2;


    dataSetServer->close(dataSet); // Finished with the file

}

/**************************************************************
*
***************************************************************/
int statistics(int nx, int ny, vector<OmSourceParameter> &source, int width, float minSig)
{

    cout << "statistcs nx=" << nx << " ny=" << ny << endl;  
    vector<int> count;
    float maxArea = float(width * width);
    for(int i = 0; i < nx; i+=width)
    {
       
        float x1 = float(i);
        float x2 = float(min(nx, i + width));
       
        for(int j = 0; j < ny; j+=width)
        {
	    int n = 0;     
            float y1 = float(j);
            float y2 = float(min(ny, j + width));
	    //            cout << "x1=" << x1 << " x2=" << x2 << " y1=" << y1 << " y2=" << y2 << endl; 
            for(unsigned k = 0; k < source.size(); k++)
	    {
	        if( source[k].significance > minSig) continue;
                if( source[k].xcoord < x1) continue;
                if( source[k].xcoord > x2) continue;
                if( source[k].ycoord < y1) continue;
                if( source[k].ycoord > y2) continue;
                ++n;


	    }
            int m = (int)ceil(n * maxArea / ((x2 - x1) * (y2 - y1)) );
            count.push_back(m); 
        }
    }
    //	    for(unsigned i = 0; i < count.size(); i++)
    //	    {
    //	      cout << i << ") count=" << count[i] << endl;

    //	    }

    sort(count.begin(), count.end() );
    int median = Median(count);
    cout << "Median count=" << median << " n=" << count.size() << endl;
    float frac = 0.05;
    int p1 = Percentile(count, frac);
    frac = 0.99;
    int p2 = Percentile(count, frac);
    cout << "0.05-0.99 percentile range=" << p1 << "-" << p2 << endl;

    return p2;
} 
/**************************************************************
*
***************************************************************/
void spuriousSources(int nx, int ny, vector<OmSourceParameter> &source, int width, float minSig, int maxCount)
{

    cout << "spurioussources nx=" << nx << " ny=" << ny << endl;  

    float maxArea = float(width * width);

    vector<OmSourceParameter> spuriousSource;
	   
    for(unsigned k = 0; k < source.size(); k++)
    {
        if( source[k].significance > minSig) continue;  
        float x1 = max(0.0, source[k].xcoord - double(width / 2));
        float x2 = min((float)nx, source[k].xcoord + float(width / 2));
     
        float y1 = max(0.0, source[k].ycoord - double(width / 2));
        float y2 = min((float)ny, source[k].ycoord + float(width / 2));
     
        int n = 0; 
        for(unsigned l = 0; l < source.size(); l++)
	{
	  if (k == l ) continue;
          
          if( source[l].significance > minSig) continue;
          if( source[l].xcoord < x1) continue;
          if( source[l].xcoord > x2) continue;
          if( source[l].ycoord < y1) continue;
          if( source[l].ycoord > y2) continue;
          ++n;


	}
        int m = (int)ceil(n * maxArea / ((x2 - x1) * (y2 - y1)) );
        if(m >= maxCount)
	{
	    OmSourceParameter theSource = source[k];
            theSource.number = k;
	    spuriousSource.push_back( theSource );

        }    

    }




   toRegion( spuriousSource, "red");

   
} 
