! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
! Omcatalogue version of omsrclistcomb
!+
! Module: omsrclistcomb_mod
!
! Authors:  Christopher Brindle - MSSL
!           Andrew King - MSSL
!           Tim Harries - MSSL
!         
! Description: Combines OSW source list into an observation source list
! 
! Comments: 
! 
! $Id: module.f90,v 1.5 1998/01/13 08:39:15 gvacanti Exp $
!
! History: 
   
!
!************************************************************************************************
module ommastersrclist_mod
  use param
  use cal
  use dal
  use Errorhandling
  use sla
  use omallocation_mod
  use omconstants_mod
  use omastromcorrect_mod
  use omutility_mod

use mastersrctable_mod
  implicit none 

  private 

  public ommastersrclist


!*****************************************
!
!****************************************
type AstrometryData

    real(single) :: xOffsetMin =  1000.0
    real(single) :: yOffsetMin =  1000.0
    real(single) :: xOffsetMax = -1000.0
    real(single) :: yOffsetMax = -1000.0
    integer(int16) :: nCorrs = 0
    real(double) :: mjd = 1e10
end type AstrometryData


!******************************************************************
! The OM filters
!******************************************************************


real(double), private :: raz, decz


integer(int32), parameter, private               :: int32NullValue = -1      
integer(int8), parameter, private               :: int8NullValue = 127      


contains 

!****************************************
! The main program
!****************************************
subroutine ommastersrclist

    integer, parameter          :: nFilt = 7             ! no of filters
    integer, parameter          :: nCol = 15             ! no of colours for instrumental magnitudes
 

    type(DataSetT)              :: oswDataSet            ! handles
    type(DataSetT)              :: dataSetH            ! handle for output dataset
    type(TableT)                :: tableSet, tableH
 

    character(1024), pointer    :: oswFile(:) => null()             ! filenames
    integer                     :: nStart                 ! start no of sources
    integer                     :: nosw                   ! no of osw lists
    integer                     :: i            ! counters
    integer                     :: nSources               ! total no of sources
    integer(int32), pointer     :: nrows(:)               ! no of rows in each osw source list
 
    type(sourceParameters1), pointer :: source(:) => null()
  

    type(AstrometryData) :: astromData

    integer :: noUVW2 = 0
    integer :: noUVM2 = 0
    integer :: noUVW1 = 0
    integer :: noU = 0
    integer :: noB = 0
    integer :: noV = 0
    real(double) :: raPoint, decPoint
    radiansToArcSeconds = radiansToDegrees * 3600.0_double


    !************************************************
    ! Get the number of osw files and their names.
    !************************************************

    nosw = parameterCount("sourcelistsets")

    if (nosw .eq. 0) then



        call warning("Can't continue", "No source-list files to process")
        stop
    end if    

 
    oswFile => AllocateChar(1024, nosw, "oswFile")


    nrows => AllocateInt32(1, nosw, "nrows")   ! The number of rows in each source-list file

    
    nSources = 0 
 
    !********************************************************************************************
    ! Determine total number of sources and number of source-list files with at least one source
    !********************************************************************************************
    nosw = 0
    do i = 1 , parameterCount("sourcelistsets")
                
        oswfile(i) = stringParameter("sourcelistsets", i - 1)
       
        oswDataSet = DataSet(oswFile(i),read)
      
        tableSet = table(oswDataSet, 0)
        nrows(i) = numberOfRows(tableSet)
    
        if (nrows(i) .gt. 0) then
            !**********************************************************************
            ! Not include filter "WHITE"
            !**********************************************************************
            if ( trim(stringAttribute(oswDataSet, "FILTER")) .ne. "WHITE") then   
                nSources = nSources + nrows(i)
                nosw = nosw + 1
                oswfile(nosw) = oswfile(i)
                nrows(nosw) = nrows(i)

               select case( trim(stringAttribute(oswDataSet, "FILTER")) )
                   case ("UVW2")
                       noUVW2 = noUVW2 + nrows(nosw)

                   case ("UVM2")
                       noUVM2 = noUVM2 + nrows(nosw)

                   case ("UVW1")
                       noUVW1 = noUVW1 + nrows(nosw)

                   case ("U")
                       noU = noU + nrows(nosw)
                   case ("B")
                       noB = noB + nrows(nosw)

                   case ("V")
                       noV = noV + nrows(nosw)
                end select


            end if
        end if
        call release(tableSet)
        call release(oswDataSet)

    end do

    !****************************************************************************************
    ! Copy the old observation source-list file to the new one
    !****************************************************************************************
    call CopyDataSet( trim(stringParameter("oldobsfile") ), trim( stringParameter("newobsfile")))   


    !*************************************
    ! Check there is at least 1 source
    !*************************************
    if (nSources .eq. 0) then
        call warning("Can't continue- no sources", "Nothing to add to new observtaion file")
    
        stop
    end if    






    !******************************************
    ! Allocate memory for the arrays to hold
    ! the input data.
    !*****************************************
 
    ! The array of structs to hold data from individual source-list files
    allocate(source(1 : nSources)) 


 
    !********************************************************
    ! Read in the data from each osw source list into the 
    ! appropriate array- appending the new data to the end of
    ! that array.
    !********************************************************
     

    write(errstr,'(a)') "Loading data from source-list files"
    call message(AppMsg, Verbose)


    nStart = 0

    raz = 0.0_double

    decz = 0.0_double

    do i = 1 , nosw

       !************************************************************************
       ! Load the data from this source-list file and append to the source-list
       !***********************************************************************
       call LoadSourceData(oswFile(i), source, nStart, astromData, raPoint, decPoint)
       nStart = nStart + nrows(i)  
       raz = raz + raPoint
       decz = decz + decPoint
 
 
    end do
    raz = raz * degreesToRadians / dble(nosw)
    decz = decz * degreesToRadians / dble(nosw)


    dataSetH = DataSet( trim(stringParameter("newobsfile")), MODIFY)

    tableH = table(dataSetH, "SRCLIST")

    !*************************************************************
    ! Add keywords for the min and max astrometry offsets
    !*************************************************************
    if (astromData%ncorrs .eq. 0) then 
        astromData%xoffsetMin = nullValue
        astromData%xoffsetMax = nullValue
        astromData%yoffsetMin = nullValue
        astromData%yoffsetMax = nullValue
    end if
 

    call setAttribute( dataSetH, "MJD", astromData%mjd, "days", "")




    nSources = numberofrows( tableH ) 
    call setAttribute( tableH, "NCORRS", astromData%ncorrs, " ", "No of source-list files corrected")
    call setAttribute( tableH, "RAOFFMIN", astromData%xoffsetMin, "arc secs", "Minimum Ra offset")
    call setAttribute( tableH, "RAOFFMAX", astromData%xoffsetMax, "arc secs", "Maximum Ra offset")
    call setAttribute( tableH, "DEOFFMIN", astromData%yoffsetMin, "arc secs", "Minimum Dec offset")
    call setAttribute( tableH, "DEOFFMAX", astromData%yoffsetMax, "arc secs", "Maximum Dec offset")
 
    call setAttribute( tableH, "UVW2_PER", real(noUVW2 * 100) / real(nsources), "%", "Percentage of UVW2 sources")
    call setAttribute( tableH, "UVM2_PER", real(noUVM2 * 100) / real(nsources), "%", "Percentage of UVM2 sources")
    call setAttribute( tableH, "UVW1_PER", real(noUVW1 * 100) / real(nsources), "%", "Percentage of UVW1 sources")
    call setAttribute( tableH, "U_PER", real(noU * 100) / real(nsources), "%", "Percentage of U sources")
    call setAttribute( tableH, "B_PER", real(noB * 100) / real(nsources), "%", "Percentage of B sources")
    call setAttribute( tableH, "V_PER", real(noV * 100) / real(nsources), "%", "Percentage of V sources")
 

    call release(dataSetH)


!    call redoAstrometry(source, oswfile(1))
    deallocate(source)
    nullify(source)



end subroutine ommastersrclist

!**********************************************************
! Add a source ID column to each of the osw source lists
! telling us which source went to which entry in the
! observation source list 
!**********************************************************
subroutine AddSourceIds(nosw, oswFile, nindep, matchedNumber, nrows, raSortIndex)
    integer, intent(in) :: nosw
    character(*), pointer :: oswFile(:)
    integer(int32), intent(in) :: nindep 
    integer, pointer :: matchedNumber(:) 
    integer, pointer :: nrows(:)
    integer, pointer :: raSortIndex(:)

    integer(int32), pointer :: sourceID(:) => null()
   
    integer :: nStart, i, j, k
    type(dataSetT) :: oswDataSet
    type(TableT) :: tableSet
    integer(int32), pointer :: tempIndex(:) => null()

  
    tempIndex => allocateInt32(1, nindep, "tempIndex")
    do i = 1, nindep
        tempIndex(raSortIndex(i)) = i
    end do 
 
    nStart = 1
 
    write(errstr, *) "Adding source ids"
    call message(AppMsg, Verbose)
          
    sourceID => allocateInt32(1, maxval(nrows), "sourceId")
  
    k = 0 ! Source number in sorted uncombined list

    do i = 1 , nosw ! Loop through each input source-list

        oswDataSet = DataSet(oswFile(i), modify)
        tableSet = table(oswDataSet, 0)
   
        do j = nStart, nStart + nrows(i) - 1    ! Add ID number to each source

            k = k + 1
            sourceID(j - nStart + 1) = tempindex(matchednumber(k))

        end do

        call AddInt32Column(tableSet, sourceId, nRows(i), &
               "SRC_ID", "", "Source number in observation source-list file")
      
        call release(tableSet)
        call release(oswDataSet)
               
        nStart = nStart + nrows(i)

    end do

    deallocate(sourceId, tempIndex)
    nullify(sourceId, tempIndex)


end subroutine  AddSourceIds


!************************************************************
! CompareOffsets
!
!************************************************************
subroutine CompareOffsets(fileName)
    character(*) :: fileName(:)
   
    real(single), pointer, dimension(:) ::  deltaRa, deltaRaError, &
                          deltaDec, deltadecError, &
                          deltaTheta, deltaThetaError
    real(single) :: deltaRamean, deltaRastandardDeviation
    real(single) :: deltaDecmean, deltaDecstandardDeviation
    real(single) :: deltaThetamean, deltaThetastandardDeviation
    real(single) :: minRaOffset, maxRaOffset
    real(single) :: minDecOffset, maxDecOffset

    character(4), pointer :: filter(:)

    logical, pointer :: status(:) => null()
    integer :: i, j, k
    integer(int8) :: noFilters
    character(4), pointer :: filterList(:), exposureNo(:)
    integer(int32), pointer :: sourceIndex(:)
    integer(int32)  :: noFiles
  
    noFiles = size(fileName)
    allocate(status(1 : noFiles))
    write(errstr, *) "Checking astrometric offsets in each source-list file"
    call message(AppMsg, Verbose)
      


    deltaRa => allocateSingle(1, noFiles, "deltaRa")
    deltaRaError => allocateSingle(1, noFiles, "deltaRa")
    deltaDec => allocateSingle(1, noFiles, "deltaRa")
    deltaDEcError => allocateSingle(1, noFiles, "deltaRa")
    deltaTheta => allocateSingle(1, noFiles, "deltaTheta")
    deltaThetaError => allocateSingle(1, noFiles, "deltaTheta")
    sourceIndex => allocateInt32(1, noFiles, "sourceIndex")

    allocate(filter(1 : noFiles))
    allocate(filterList(1 : noFiles))
    allocate(exposureNo(1 : noFiles))
    

    k = 0
    minRaOffset = 1e6
    maxRaOffset = -1e6
    minDecOffset = 1e6
    maxDecOffset = -1e6

    noFilters = 0
iLoop : do i = 1, noFiles
 
        call GetOffsets(trim(fileName(i)), deltaRa(i), deltaRaError(i), &
                        deltaDec(i), deltadecError(i), &
                        deltaTheta(i), deltaThetaError(i), filter(i), exposureNo(i), status(i))
        if (status(i)) then
            k = k  + 1
            deltaRa(k) = deltaRa(i)
            deltaRaError(k) = deltaRaError(i)
            deltaDec(k) = deltaDec(i)
            deltaDecError(k) = deltaDecError(i)
            deltaTheta(k) = deltaTheta(i)
            deltaThetaError(k) = deltaThetaError(i)
            minRaOffset = min(minRaOffset,  deltaRa(k))
            maxRaOffset = max(maxRaOffset,  deltaRa(k))
            minDecOffset = min(minDecOffset,  deltaDec(k))
            maxDecOffset = max(maxDecOffset,  deltaDec(k))
            filter(k) = filter(i) 
            exposureNo(k) = exposureNo(i)
            do j = 1, noFilters 
                if (trim(filter(i)) == trim(filterList(j))) cycle iLoop
            end do
            noFilters = noFilters + 1
            filterList(noFilters) = trim(filter(i))
                    
        end if 
       
    end do iLoop 
    do i = 1, k
        
        do j = 1, noFilters 
            if (trim(filter(i)) == trim(filterList(j))) then
                sourceIndex(i) = j
                exit
            end if
        end do 
    end do 
    if (k > 1) then


        call ComputeAverage(deltaRa, deltaRaError, k, deltaRamean, deltaRastandardDeviation)
        call ComputeAverage(deltaDec, deltaDecError, k, deltaDecmean, deltaDecstandardDeviation)
        call ComputeAverage(deltaTheta, deltaThetaError, k, deltaThetamean, deltaThetastandardDeviation)
  
       
        write(errstr, '(a, i3)') "No of computed astrometric offsets=", k
        call message(AppMsg, Verbose)
    
        write(errstr, '(a, f6.2, a, f6.2)') "Mean RA offset=", deltaRamean, "+/-", deltaRastandardDeviation
        call message(AppMsg, Verbose)
        write(errstr, '(a, f6.2, a, f6.2)') "Mean dec offset=", deltaDecmean, "+/-", deltaDecstandardDeviation
        call message(AppMsg, Verbose)
        write(errstr, '(a, f6.2, a, f6.2, a)') "Ra offset range=", minRaOffset, "-",  maxRaOffset, " arcsecs"
        call message(AppMsg, Verbose)
        write(errstr, '(a, f6.2, a, f6.2, a)') "Dec offset range=", minDecOffset, "-",  maxDecOffset, " arcsecs"
        call message(AppMsg, Verbose)
  
    end if

    deallocate(deltaRa, deltaRaError, deltaDec, deltaDecError, deltaTheta, deltaThetaError) 
    deallocate(filter, filterList, exposureNo, sourceIndex, status)
end subroutine CompareOffsets    
!*****************************************************
! Get the offsets from teh source-list file        
!*****************************************************
subroutine GetOffsets(sourceList, deltaRa, deltaRaError, &
                        deltaDec, deltadecError, &
                        deltaTheta, deltaThetaError, filter, exposureNo, status)
    character(*), intent(in) :: sourceList
    real(single), intent(out) ::  deltaRa, deltaRaError, &
                          deltaDec, deltadecError, &
                          deltaTheta, deltaThetaError
    character(*), intent(inout) :: filter, exposureNo
    character(20) :: text
    logical, intent(out) :: status
    type(dataSetT) :: dataSetH
    type(tableT)   :: sourceTable
      dataSetH = dataSet(sourcelist, read)

    filter = trim(stringAttribute(dataSetH, "FILTER")) !(11 : 13) 
       
    text = trim(stringAttribute(dataSetH, "EXP_ID")) !(11 : 13) 
    exposureNo = text(11 : 13)
 
 
    sourceTable = table(dataSetH, "SRCLIST")
    if (.not. hasAttribute(sourceTable, "RAOFFSET")) then
        status = .false.
        call release(sourceTable) 
        call release(dataSetH)
 
        return
    end if 
    
    if (.not. booleanAttribute(sourceTable, "POSCOROK")) then
        status = .false.
        call release(sourceTable) 
        call release(dataSetH)
 
        return
    end if 
    

    deltaRa = real32Attribute(sourceTable, "RAOFFSET")
    deltaRaError = real32Attribute(sourceTable, "RAOFFERR")
    deltaDec = real32Attribute(sourceTable, "DEOFFSET")
    deltaDecError = real32Attribute(sourceTable, "DEOFFERR")
    deltaTheta = real32Attribute(sourceTable, "ROT_CORR")
    deltaThetaError = real32Attribute(sourceTable, "ROT_ERR")
    


    call release(sourceTable) 
    call release(dataSetH)
    status = .true.
   
end subroutine GetOffsets



!********************************************************
!
!
!*******************************************************
subroutine LoadSourceData(oswFile, source, nStart, astromData, raPoint, decPoint)

    character(*), intent(in) :: oswFile
    type(sourceParameters1), pointer :: source(:)
    integer, intent(in) :: nStart
    type(astrometryData), intent(inout) :: astromData
    

    type(TableT) :: tableSet
    type(DataSetT) :: oswDataSet
    integer :: nRows
    integer(int32) :: filterIdInt
    real(double) :: plateScale
    character(4) :: omFilter
    real(single) :: exposure
    real(double), pointer, dimension(:) :: temp => null()

    real(double), intent(out) :: raPoint, decPoint

    integer :: windowX0, windowY0, xbin, ybin

    oswDataSet = DataSet(oswFile, read)
    tableSet = table(oswDataSet, 0)
    nrows = numberOfRows(tableSet)

    exposure =  real32Attribute(oswDataSet, "EXPOSURE") ! The exposure time (secs)

 

    source(nStart + 1 : nStart + nrows)%exposure = exposure
   

    omFilter = stringAttribute(oswDataSet, "FILTER")



     call GetWindowParameters(oswDataSet, windowX0, windowY0, xbin, ybin)



    !*********************************************************************************************
    ! Get the astrometry data
    !********************************************************************************************
!    if ( booleanattribute( tableset, "POSCOROK") ) then
!        astromData%xoffsetmin = min(astromdata%xoffsetmin, real32attribute( tableset, "RAOFFSET") )
!        astromData%xoffsetmax = max(astromdata%xoffsetmax, real32attribute( tableset, "RAOFFSET") )

!        astromData%yoffsetmin = min(astromdata%yoffsetmin, real32attribute( tableset, "DEOFFSET") )
!        astromData%yoffsetmax = max(astromdata%yoffsetmax, real32attribute( tableset, "DEOFFSET") )
!        astromData%ncorrs = astromData%ncorrs + 1
!    end if



    !  Get the filter Id integer value
    filterIdInt = filterIdToInt(omFilter)

    call CAL_setState(OM, filterId = filterIdInt)


    ! Get the Plate scale (arcsecs/pixel)
    call CAL_omGetPlateScale(0.55_double, plateScale)


   
    ! Get the position error
    call ReadReal32Column(tableSet, "POSERR", nrows, nStart, source%poserr)

    !***********************************************************************
    ! Convert position to arc secs
    !********************************************************************** 
    source(nStart + 1 : nstart + nrows)%poserr = &
               real(dble(source(nstart + 1 : nstart + nrows)%poserr) * plateScale * dble(xbin))

  
    ! Get the Right-Ascension and declination
    
    temp => allocateDouble( 1, nrows, "temp")

    call ReadReal64Column(tableSet, "RA", nrows, 0, temp)
    source(nstart + 1 : nStart + nrows)%rightascension = temp

    call ReadReal64Column(tableSet, "DEC", nrows, 0, temp)

    source(nstart + 1 : nStart + nrows)%declination = temp
    

    deallocate(temp)
    nullify(temp)
   





    ! Get the position error
    call ReadReal32Column(tableSet, "POSERR", nrows, nStart, source%poserr)


    !***********************************************************************
    ! Convert position to arc secs
    !**********************************************************************
    source(nStart + 1 : nstart + nrows)%poserr = &
               real(dble(source(nstart + 1 : nstart + nrows)%poserr) * plateScale * dble(xbin))






    ! Get the instrumental magnitudes
    if ( hasColumn( tableSet, "MAG") ) then 
        call ReadReal32Column(tableSet, "MAG", nrows, nStart, source%magnitude)
        call ReadReal32Column(tableSet, "MAGERR", nrows, nStart, source%magnitudeerr)
    end if


    if ( hasColumn( tableSet, "SCREENING_COMMENT" ) ) then
        call ReadCharColumn(tableSet, "SCREENING_COMMENT", nrows, nStart, source%comment)
    end if

  
  
    source(nstart + 1: nstart + nrows)%filter = omFilter
    source(nstart + 1: nstart + nrows )%mjd = real64Attribute( oswdataset, "MJDREF") 
 

    raPoint = real64attribute( oswDataSet, "RA_PNT") 
    decPoint = real64attribute( oswDataSet, "DEC_PNT")


    astromdata%mjd = min( astromdata%mjd, ModifiedJulianDate( stringAttribute(oswdataSet, "DATE-OBS" )))
  
    call release(oswDataSet)





    !*****************************************************************************
    ! Do an astrometry correction (don't actuall correct the sources)
    !****************************************************************************        
    call SourceListAstrometry( oswfile, raPoint, decPoint,   source(nstart + 1 : nStart + nrows), astromData )


    

       
end subroutine LoadSourceData



!****************************************************************************
!
!***************************************************************************

subroutine SourceListAstrometry( file, raz, decz, source, astromData)
    character(*), intent(in) :: file
    real(double), intent(in) :: raz, decz
    type(sourceParameters1) :: source(:)
    type(AstrometryData), intent(inout) :: astromData

    type(sourceParameters), pointer :: source1(:) => null()


    logical :: status
    type(correctionData) :: corrData
    integer(int32) :: nSources  



return

    if ( .not. SetExists(stringParameter("catfile"))) return


    call setCorrectPositions( .false. )
    call setminNosources( 4 ) 





    allocate( source1 ( 1 : size(source)))
    call convert2( source, source1 )

    nsources = size(source)

    source1%rightAscension = source1%rightAscension * degreesToRadians
    source1%declination = source1%declination * degreesToRadians






    call CorrectAstrometry( nSources, source1%rightascension, source1%declination,   &
                           source1%posErr, source1%magnitude, &
                           raz * degreesToRadians,            &
                           decz * degreesToRadians,           &
                           trim(file),   &
                           status, corrData)     

    deallocate( source1 )
    nullify( source1 )



    if (status) then




        astromData%xoffsetmin = min(astromdata%xoffsetmin, corrData%xShift )
        astromData%xoffsetmax = max(astromdata%xoffsetmax, corrData%xShift )

        astromData%yoffsetmin = min(astromdata%yoffsetmin, corrData%yShift )
        astromData%yoffsetmax = max(astromdata%yoffsetmax, corrData%yShift )
        astromData%ncorrs = astromData%ncorrs + 1



    end if



end subroutine SourceListAstrometry

!****************************************************************
!
!**************************************************************
subroutine redoAstrometry(source, file)


    type(sourceParameters1), pointer :: source(:)
    character(*), intent(in) :: file
    type(sourceParameters), pointer :: source1(:) => null()



    logical :: status, oldposcorrok
    type(correctionData) :: corrData, olddata

    type(dataSetT) :: olddataSetH
    character(200) :: oldObsFile
    type(tableT) :: oldtableH 
    oldObsFile = trim(stringParameter("oldobsfile"))

    oldDataSetH = dataSet( oldobsfile, READ) 
    oldTableH = table(oldDataSetH, "SRCLIST", read)

    allocate( source1 ( 1 : size(source)))
    call convert2( source, source1 )


    source1%rightAscension = source1%rightAscension * degreesToRadians
    source1%declination = source1%declination * degreesToRadians


write(*, *) "Line 778"
    call CorrectAstrometry( size(source1) , source1%rightascension, source1%declination,   &
                           source1%posErr, source1%magnitude, &
                           raz,                               &
                           decz,                              &
                           file,   &
                           status, corrData)     


write(*, *) "Line 787"




    deallocate( source1 )
    nullify( source1 )



    if (status) then
        !******************************************************************
        ! Compare with original offsets
        !*****************************************************************
write(*, *) "Line 801"


        if ( hasAttribute( oldtableH, "RAOFFSET" ) )then

            olddata%xshift = real32attribute( oldtableH, " RAOFFSET")
            olddata%sigxshift = real32attribute( oldtableH, "RAOFFERR")
            olddata%yshift = real32attribute( oldtableH, "DEOFFSET")
            olddata%sigyshift =  real32attribute( oldtableH, "DEOFFERR")
            olddata%rmsresidual = real32attribute( oldtableH, "RMSRESID")   
            olddata%nomatched =  int32attribute( oldtableH, "NMATCHES")
            oldposcorrok =  booleanattribute( oldtableH, "POSCOROK")




            if (oldposcorrok) then



                write(errstr,'(a, f6.2, a, f6.2, a, f6.2, a, f6.2)') &
                      "Original ra shift=", olddata%xshift, "+/-", olddata%sigyshift, &
                      " new=", corrdata%xshift, "+/-", corrdata%sigyshift

                call message(AppMsg, Verbose)
                write(*,'(a, f6.2, a, f6.2, a, f6.2, a, f6.2)') &
                      "Original ra shift=", olddata%xshift, "+/-", olddata%sigyshift, &
                      " new=", corrdata%xshift, "+/-", corrdata%sigyshift



                write(errstr,'(a, f6.2, a, f6.2, a, f6.2, a, f6.2)') &
                      "Original dec shift=", olddata%yshift, "+/-", olddata%sigyshift, &
                      " new=", corrdata%yshift, "+/-", corrdata%sigyshift
                call message(AppMsg, Verbose)

                write(*,'(a, f6.2, a, f6.2, a, f6.2, a, f6.2)') &
                      "Original dec shift=", olddata%yshift, "+/-", olddata%sigyshift, &
                      " new=", corrdata%yshift, "+/-", corrdata%sigyshift
          

                write(errstr,'(a, f6.2, a, f6.2, a, f6.2, a, f6.2)') &
                      "Original rmsresidual=", olddata%rmsresidual, " new=", corrdata%rmsresidual
                call message(AppMsg, Verbose)

                write(*,'(a, f6.2, a, f6.2, a, f6.2, a, f6.2)') &
                      "Original rmsresidual=", olddata%rmsresidual, " new=", corrdata%rmsresidual
          

                 write(*,'(a, i6, a, i6)') &
                      "Original no matched=", olddata%nomatched, " new=", corrdata%nomatched
          



                if ( abs( olddata%xshift - corrdata%xshift) .gt. 1.0 .or. &
                     abs( olddata%yshift - corrdata%yshift) .gt. 1.0) then

!                    call fatal("bug in astrometry correction")
                end if 

            end if

 



        end if
    endif

    call release(olddataSetH)

end subroutine redoAstrometry

end module ommastersrclist_mod









